/******
        file     : shared_motifs.h
        function : headers for all alignment needed
                                                 
        created  : 26-29 Nov 2002
        modif    : 6 mar 2003

        author   : amikezor
*****/

#ifndef _ALIGN_H_ 
#define _ALIGN_H_ 1

#define MAX2( A, B )     ( ((A)>(B))?(A):(B) )
#define MIN2( A, B )     ( ((A)>(B))?(B):(A) )
#define MAX3( A, B, C )  ( ((MAX2(A,B))>(C))?(MAX2(A,B)):(C) )


#define CHAR2SYMB( A ) ( (A) - 65 )
#define SYMB2CHAR( i ) ( (i) + 65 )

#define Scores(A,B) pResults->scores[  (A)*pResults->size_col + (B) ]
#define Traces(A,B) pResults->traces[  (A)*pResults->size_col + (B) ]
#define Match(A,B) pScoring->matrix[ CHAR2SYMB(seq1[ (A)-1 ]) ][ CHAR2SYMB(seq2[ (B)-1 ]) ]

#define Gap_above(A,B) pResults->F[  (A)*pResults->size_col + (B) ]
#define Gap_left(A,B) pResults->E[  (A)*pResults->size_col + (B) ]

#define MATRIX_SIZE 26

typedef struct{

	int matrix[MATRIX_SIZE][MATRIX_SIZE];  /* the matrix of matc/missmatch */
	int gap_open;                          /* value of gap-open */
	int gap_ext;                           /* value of gap_ext */

} SCORING;

typedef struct{

	int *scores;          /* the score matrix */
	int *F;               /* needed for memorizing deletion in seq2 (col=col;row++) */
	int *E;
	char *traces;         /* the way matrix - could take values 'l'eft, 'd'iagonal, or 'a'bove or 'u'nknown */
	
	
	int size_row;         /* size of the matrices */
	int size_col;
	
	int BestScore;       /* best score and its position in matrix */
	int BestScore_row;
	int BestScore_col;   

	char *traceback;     /* all you need for bactracking */

	int Begin_row;       /* The begin of the alignemnt - seq2 */
	int Begin_col;       /* The begin of the alignemnt - seq1 */
	int alignment_len;   /* guess ?? */
	int matches;         /* number of match (score>0 in scoring matrix) */

} RESULTS;

/*
	in waterman_eggert.c
*/
void waterman_eggert(char *seq1, char *seq2, SCORING *pScoring, RESULTS *pResults, int round, short verbose);


/*
	in traceback_SW.c
*/
void traceback_SW( char *seq1, char *seq2, SCORING * pScoring,  RESULTS *pResults , char direction);

/*
	in matrix.c and matrixlog.c
*/
void identity_matrix(SCORING *pScoring, int gap_open, int gap_ext);
void matrix_log(char *sequence, SCORING *pScoring, int gap_open, int gap_ext);

/*
	in mem.c
*/
void mem_RecursivAlign(RESULTS *pResults, char *seq1, char *seq2);
void free_RecursivAlign(RESULTS *pResults);
void memset_RecursivAlign(RESULTS *pResults, char *seq1, char *seq2);

/*
	in explore2ways.c
*/
void explore2ways(char *seq1, char *seq2, SCORING *pScoring,  RESULTS *pResults);

/*
	in ouput.c
*/
void print_matrices( RESULTS Results, char *seq1, char *seq2 );
void print_Scoring( SCORING Scoring );
void print_alignment( RESULTS *pResults, SCORING *pScoring, char *seq1, char *seq2 );
void print_shared_motif( RESULTS *pResults, SCORING *pScoring, char *type );
void print_dotplot( RESULTS *pResults , char *type);

/*
	in checkseq.c
*/
short CheckSeq(char *seq, char *alpha);
void nonACGTXtoN(char *seq);
void UpperSequence(char *seq);


/*
	in tableR.c
*/
void mem_tableR(int size_seq1, int size_seq2, int **ptableR1, int **ptableR2);
void free_tableR(int *tableR1, int *tableR2);
void Update_tableR(int *tableR1, int *tableR2, int Pos1, int Len1, int Pos2, int Len2 );
void Finish_tableR(int size_seq1, int size_seq2, int *tableR1, int *tableR2);

/*
	In lmin_2seqs.c
*/
double KLminRiskSeq(char *sequence, int r, float p, long size1, long Nx);

/*
	In readfst.h
*/
char *file2mem(char *file, long *size);
void invseq(char *seqsrc, char *seqdest);


#endif
