/******
        file     : output.c
        function : all output for alignemnt
                                                 
        created  : 26 Nov 2002
        modif    : 2 Apr 2003 - add print_dotplot( )

        author   : amikezor
*****/

#include "SharMot.h"
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#define OUTPUT_SIZE 60

void static print_nchar(char *string, int n){

	char tmp;
	
	tmp = string[n];
	string[n]=0;
	printf("%s", string);
	string[n]=tmp;

}

int static count_del(char *string, int n){

	int i;
	int del=0;
	
	for(i=0;i<n;i++)
		if(string[i]=='-')
			del++;

	return del;
}

void print_shared_motif( RESULTS *pResults, SCORING *pScoring, char *type ){
	
	if( ! strcmp(type,"direct") ){

		printf("direct  \t%d\t%d\t%d\t%d\t%d\t%d\n",
		pResults->Begin_row,
		pResults->Begin_col,
		(pResults->BestScore_row-pResults->Begin_row+1),
		(pResults->BestScore_col-pResults->Begin_col+1),
		pResults->matches,
		pResults->BestScore
		);

	}
	else if
	( ! strcmp(type,"inverted") ){

		printf("inverted\t%d\t%d\t%d\t%d\t%d\t%d\n",
		pResults->Begin_row,
		(pResults->size_col-pResults->BestScore_col ),  /* len seq - bestcol */
		(pResults->BestScore_row-pResults->Begin_row+1),
		(pResults->BestScore_col-pResults->Begin_col+1),
		pResults->matches,
		pResults->BestScore
		);

	}
	return;
}


void print_alignment( RESULTS *pResults, SCORING *pScoring, char *seq1, char *seq2 ){

	char *align1,
	     *align2,
		  *identity;
	
	int n1 = pResults->Begin_row-1;
	int n2 = pResults->Begin_col-1;
	int n=0;
	
	int count;
	int del;
	
	printf("Score: %d ; Seq1 [%d..%d] ; Seq2 [%d..%d] ; Alignment_length: %d ; Matches: %d\n",pResults->BestScore,
		pResults->Begin_row, pResults->BestScore_row,
		pResults->Begin_col, pResults->BestScore_col,
		pResults->alignment_len, pResults->matches);
	
	/*
		Get memory
	*/
	align1  =  (char *) malloc( (1 + pResults->alignment_len)*sizeof(char) );
	align2  =  (char *) malloc( (1 + pResults->alignment_len)*sizeof(char) );
	identity = (char *) malloc( (1 + pResults->alignment_len)*sizeof(char) );
	if(!align1 || !align2 || !identity)fprintf(stderr,"print_alignment: malloc error\n"),exit(3);
	
	align1[n]=seq1[n1];
	align2[n]=seq2[n2];
	identity[n]='|';
	
	/*printf("len of traceback is %ld\n%s\n",strlen(pResults->traceback),pResults->traceback); */
	
	for( count=pResults->alignment_len-1 ; count >=0 ; count --){
		
		if( pResults->traceback[count] == 'd' ){
			align1[n]   = seq1[n1];
			align2[n]   = seq2[n2];
			identity[n] = ( seq1[n1]==seq2[n2]  )?'|': ' ';   /*  is space  */
			
			n1++;
			n2++;
			n++;
		}
		else if( pResults->traceback[count] == 'a'){
			align1[n]   = seq1[n1];
			align2[n]   = '-';
			identity[n] = ' ';   /* \32 is space */
			
			n1++;
			n++;
		}
		else{
		
			align1[n]   = '-';
			align2[n]   = seq2[n2];
			identity[n] = ' ';   /* \32 is space */
			
			n2++;
			n++;
		}
		
	}
	
	align1[n]=align2[n]=identity[n]=0;

	n=0;
	n1=pResults->Begin_row;
	n2=pResults->Begin_col;
	
	while( n < pResults->alignment_len ){

		if( (pResults->alignment_len - n) > OUTPUT_SIZE)
		{
			printf("%5d ",n1);                               /* print the first seq */
			print_nchar(align1+n, OUTPUT_SIZE);
			
			del = count_del(align1+n, OUTPUT_SIZE);
			n1 = n1+(OUTPUT_SIZE)-del;                     /* get where we are in seq1 */
			
			printf(" %-5d\n      ",n1-1);
			print_nchar(identity+n, OUTPUT_SIZE);            /* print identity line */
		
			printf("\n%5d ", n2);                            /* print the second seq */
			print_nchar(align2+n, OUTPUT_SIZE);
			
			del = count_del(align2+n, OUTPUT_SIZE);         /* get where we are in seq2 */
			n2 = n2+(OUTPUT_SIZE)-del;
			
			printf(" %-5d\n\n", n2-1);                       /* print the second psoition */

		}
		else{
		
			printf("%5d %s",n1, align1+n);

			del = count_del(align1+n, strlen(align1+n));
			n1 = n1+(int)strlen(align1+n)-del;

			printf(" %-5d\n      %s\n%5d %s",
			         n1-1,
						identity+n,
						n2,
						align2+n);
						
			del = count_del(align2+n, strlen(align2+n));
			n2 = n2+(int)strlen(align2+n)-del;
			
			printf(" %-5d\n", n2-1);
		}
		
		n+=OUTPUT_SIZE;
	}

	printf("//\n");

	free(align1);
	free(align2);
	free(identity);

}

void print_dotplot( RESULTS *pResults , char *type){
	
	int n1=0;
	int n2=0;
	int count;
	int token;
		
	if( ! strcmp(type,"direct") ){

		n1 = pResults->BestScore_row;
		n2 = pResults->BestScore_col;
		token=1;
		 
	}
	else if
	( ! strcmp(type,"inverted") ){

		n1 = pResults->BestScore_row;
		n2 = pResults->size_col-pResults->BestScore_col;
		token=-1;
	}
	else
		fprintf(stderr,"print_dotplo: type is either direct or inverted, bye\n"),exit(4);
		

	for( count=pResults->alignment_len-1 ; count >=0 ; count --)
		if( pResults->traceback[count] == 'd' ){
			printf("dotplot\t%s\t%d\t%d\n",type, n1,n2);
			n1--;
			n2-=token;
		}
		else if( pResults->traceback[count] == 'a')
			n1--;
		else
			n2-=token;

}



/*
	print the scoring matrix
*/
void print_Scoring(SCORING Scoring){

	short i,u;

	printf("*******\nmatrix:\n    ");
	for(i=0;i<MATRIX_SIZE;i++)
		printf("%4c",SYMB2CHAR(i));
	printf("\n");
		
	for(i=0;i<MATRIX_SIZE;i++, printf("\n") ){

		printf("%4c",SYMB2CHAR(i));
		for(u=0;u<MATRIX_SIZE;u++)
			printf("%4d",Scoring.matrix[i][u]);
	}

	printf("gap_open: %d\ngap_ext: %d\n*******\n", Scoring.gap_open, Scoring.gap_ext);
}

#undef Scores 
#undef Traces

#define Scores(A,B) Results.scores[  (A)*Results.size_col + (B) ]
#define Traces(A,B) Results.traces[  (A)*Results.size_col + (B) ]

void print_matrices(RESULTS Results, char *seq1, char *seq2){

	int col, row;

	long size1=strlen(seq1)+1,
	     size2=strlen(seq2)+1;

	for(row=0;  row<size1;  row++, printf("\n"))
		for(col = 0;  col <size2;  col++)
			printf("%5d",Scores(row,col));
	printf("\n\n");

	for(row=0;  row<size1;  row++, printf("\n"))
		for(col = 0;  col <size2;  col++)
			printf("%5c",Traces(row,col)?Traces(row,col):'0');
			

	printf("Best score is %d at position [%d,%d]\n",
	Results.BestScore,Results.BestScore_row,Results.BestScore_col);


}
#undef Scores 
#undef Traces

#undef OUPUT_SIZE
