/******
        file     : waterman_eggert.c
        function : align 2 sequence by Water and Eggert 1987
                                                 
        created  : 6 Mar 2003
		  
        modif    : 07 march 03, correct a small bug at the init phase of fill_matrices()
		                         (it was for(row=0;  row < size2;...  )
        
		           * 10 march 03, correct a bug in the refill. Should explore until row>=Min_row or col>=Min_col
		                         (where it stops at the previous line) -- correct a small bug in that routine.
										 
		           * 20 march 03, decided to say in case of equal scores, let say it is Diagonal that wins
						             "if( Diag >= Gap_left(row,col) && Diag >= Gap_above(row,col) )", then it is Diag

		           * 07 may 03, change the whole Change_matrices, function allowing to diag if there is no score improvment
						            this new version allow to keep all segt, but avoid to refind an old aligment 

		           * 26 nov 03, 1> change Change_matrices(): add the variable trace 
					               2> correct 2 bugs in refill_matrices();
					               3> change refill_matrices() in order to reduce the space; add variables LastCol, LastRow

        author   : amikezor
*****/

#include "SharMot.h"
#include <stdio.h>
#include <string.h>
#include <stdlib.h>


/*
	In that algorithm, you do not want to rediscover an old already aligned segment,
	thus left and above gaps are recalculated, and NO INCREASE by Diag are allowed
	if 'tag_diag' is set to 0, we cannot make a move in diagonal (i.e. last-aligned segment)
	return 0 if both matrix are already identical
	else return 1 -- some change have been made
*/
static short Change_Matrices( char *seq1, char *seq2, SCORING *pScoring, RESULTS *pResults, int row, int col, short tag_diag){
		
	int Left=0,
		Above=0,
		Diag=0;
		
	int score;
	char trace;
		
	Left  = MAX3( 0,  Scores(row,col-1)+pScoring->gap_open ,  Gap_left(row,col-1)+pScoring->gap_ext );
	Above = MAX3( 0,  Scores(row-1,col)+pScoring->gap_open  , Gap_above(row-1,col)+pScoring->gap_ext );
	
	if(tag_diag){
		Diag = MAX2( Scores( row-1, col-1 )+ Match(row, col) , 0);
		Diag = MIN2( Scores( row, col) , Diag);                     /* to avoid refinding an old alignt, you cannot */
	}                                                              /* increase your score in a refill */
	                                                               
	/*
		Compare the scores and reset the score
	*/
	if( Diag >= Left  &&  Diag >=  Above){
		score = Diag;                     /* diag is the best */
		trace = 'd';            /* set the last move to diag */ 
	}
	else if( Left >  Above  ){
		score = Left;                     /* left is sup to above */
		trace = 'l';            /* set the last move to left */ 
	}
	else if(  Above  >  Left ){
		score = Above;                    /* Above is better */
		trace = 'a';            /* thus we come from above */
	}
	else{
		score =  Above;                   /* unknown : we don't know: both way are equivalent */
		trace = 'u';
	}
	
	/*
		If it everything is equal, the rest of the line also, just break;
	*/
	if( Left==Gap_left(row,col) && Above==Gap_above(row,col) && score==Scores(row,col) && trace==Traces(row,col))
		return 0;
			
	/*
		Set the new matrices (Gaps and Scores)
	*/
	Gap_above(row,col) = Above;
	Gap_left(row,col) = Left;
	Scores(row,col) = score;
	Traces(row,col) = trace;

	return 1;
}


/*
	This function refill the matrix after an alignment
	This is the tricky part of the recursive alignment
	
	Inspired by Waterman and Eggert 1987
*/
void static refill_matrices(char *seq1, char *seq2, SCORING *pScoring, RESULTS *pResults){

	int col,	row;                /* current row and col */
		
	int aligned_row,             /* starting point for each refill -- typically it follows the alignent */
		 aligned_col;
		 
	int LastRow, LastCol;        /* where the last col/row changed this current run of exploration */
	
	int Min_row, Min_col;        /* where the exploration HAS to go - it is LastRow/Col of the last run of exploration */	

	int size1= (int) strlen(seq1)+1,
		 size2= (int) strlen(seq2)+1;

	int diag=0;            /* used in the second part, after the whole alignment has been scanned to tag if the current score has changed */

	char *traces=pResults->traceback+strlen(pResults->traceback)-1;

		

	Min_col=Min_row=0;

	aligned_row=pResults->Begin_row;   /* set the begining of alignemnt  */
	aligned_col=pResults->Begin_col;
	

	/*
		First reset the alignment, right and below
	*/

	while( traces >= pResults->traceback ){

		row = aligned_row;               /* set the anchor on the alignment */
		col = aligned_col; 

		LastCol=aligned_col;
		LastRow=aligned_row;
			
		if(*traces == 'd'){
			
			/*
				Change here, then right (row)
			*/
			
			Change_Matrices( seq1, seq2,pScoring, pResults, aligned_row, aligned_col, 0);  /* no diag match/missmatch is possible here */
					
			col=aligned_col+1;
			while( col<size2  && ( (Change_Matrices(seq1, seq2,pScoring, pResults, aligned_row, col, 1) && (LastCol=col) ) || col<=Min_col) ) 
				col++;
				
         /*
				Then Change below || column
			*/
			row = aligned_row+1; 
			while( row<size1 && ( (Change_Matrices(seq1, seq2,pScoring, pResults, row, aligned_col, 1) && (LastRow=row) ) || row<=Min_row) )
				row++;
						
			Min_row=LastRow;
			Min_col=LastCol;
			
			aligned_row++;
			aligned_col++;
			
		}
		if(*traces == 'a'){
			/*
				Change here, then right
			*/
			col=aligned_col;
			
			
			while( col<size2 && ( (Change_Matrices(seq1, seq2,pScoring, pResults, aligned_row, col, 1) && (LastCol=col) )|| col<=Min_col) )
				col++;
			
			Min_col=LastCol;
			aligned_row++;
			
		}
		if(*traces == 'l'){
		
         /*
				Change here then below
			*/
			row = aligned_row; 
			while( row<size1 && ( (Change_Matrices(seq1, seq2,pScoring, pResults, row, aligned_col, 1) && (LastRow=row)) || row<=Min_row) )
				row++;
			
			Min_row=LastRow;
			aligned_col++;
			
		}
				
		traces--;
	}
	

	
	/*
		And then reset what is following
	*/

	aligned_row=pResults->BestScore_row+1; 
	aligned_col=pResults->BestScore_col+1;
	
	while( aligned_row<size1 && aligned_col<size2 ){
		
		row = aligned_row;               /* set the anchor on the diagonal */
		col = aligned_col; 

		LastRow=row;
		LastCol=col;

		/*
	 		Change here
		 */
		diag = Change_Matrices( seq1, seq2,pScoring, pResults, row, col,1 );

		/*
			Change right
		*/		
		col = aligned_col+1;
		while( col<size2 && ( (Change_Matrices(seq1, seq2,pScoring, pResults,  aligned_row, col, 1) && (LastCol=col) ) || col<=Min_col ) )
			col++;
		
		/*
			Change below
		*/
		row = aligned_row+1; 
		while( row<size1 && ( (Change_Matrices(seq1, seq2, pScoring, pResults, row, aligned_col, 1) && (LastRow=row) ) || row<=Min_row )   )
			row++;

		if(row == aligned_row+1 && col == aligned_col+1 && !diag)
			break;                 /* In that case, there is no need to go on */

		Min_row=LastRow;
		Min_col=LastCol;
		
		aligned_col++;
		aligned_row++;
	}

}



/*
	That function fill up the matrices
	First set first row, then first col then fill the entire matrix
*/
void static fill_matrices(char *seq1, char *seq2, SCORING *pScoring, RESULTS *pResults){
	
	int col, row;    /* current row and col */
		
	int size1= (int) strlen(seq1)+1,
		 size2= (int) strlen(seq2)+1;

	int Diag;                  /* score	coming from diag */
		
	pResults->BestScore=Scores(0,0);  /* init the best score */
	pResults->BestScore_row=0;
	pResults->BestScore_col=0;
	
	/*
		First fill up the first col
	*/
	for(col=0;  col < size2;  col++){
		Scores( 0, col)=0;
		Traces(0,col)= 'd';
		Gap_above(0,col)=0;             /* set the whole int *F array to zero */
	}
	/*
		Then fill up the first row
	*/
	for(row=0;  row < size1;  row++){
		Scores( row, 0 ) = 0;
		Traces( row, 0 ) = 'd';
		Gap_left(row,0) = 0;             /* set the whole int *E array to zero */
	}

	
	/*
		Then fill up the whole matrix
	*/	
	for(row = 1;  row < size1;  row++)
		for( col=1;  col < size2;  col++){
					 
			/*
				Get the three possible scores
			*/
			
			 Gap_left(row,col) = MAX3( 0,  Scores(row,col-1)+pScoring->gap_open ,  Gap_left(row,col-1)+pScoring->gap_ext );
			 Gap_above(row,col) = MAX3( 0,  Scores(row-1,col)+pScoring->gap_open ,  Gap_above(row-1,col)+pScoring->gap_ext );
			 Diag = MAX2( Scores( row-1, col-1 )+ Match(row, col) , 0);

			/*
				Compare the scores
			*/
			if( Diag >= Gap_left(row,col) && Diag >= Gap_above(row,col) ){
				Scores(row,col) = Diag;                  /* diag is eq or sup to others */
				Traces(row,col) = 'd';                   /* thus coming from diag */ 
			}
			else if( Gap_left(row,col) > Gap_above(row,col) ){
				Scores(row,col) = Gap_left(row,col);   /* left is sup to above */
				Traces(row,col) = 'l';                 /* set the last move to left */ 
			}
			else if( Gap_above(row,col) > Gap_left(row,col) ){
				Scores(row,col) = Gap_above(row,col);          /* Above is better */
				Traces(row,col) = 'a';                 /* thus we come from above */
			}
			else{
				Scores(row,col) = Gap_above(row,col);          /* unknown : we don't know: both way are equivalent */
				Traces(row,col) = 'u';
			}
			
			
			/*
				There is a Higher Score
			*/
			if(Scores(row,col) > pResults->BestScore){
				 pResults->BestScore = Scores(row,col);    /* memorize the best score */
				 pResults->BestScore_row = row;
				 pResults->BestScore_col = col;
			}
			
			
			/*
				The Higher Score is present twice, thus choose as the
				Waterman and Eggert - 1987 - the smallest way
			*/
			if(Scores(row,col) == pResults->BestScore){
				 if(row+col < pResults->BestScore_row+pResults->BestScore_col ||
				      (row+col == pResults->BestScore_row+pResults->BestScore_col &&
						 row<pResults->BestScore_row)  )
				{
				 	pResults->BestScore = Scores(row,col);  
				 	pResults->BestScore_row = row;
				 	pResults->BestScore_col = col;
					
				 }
			}
			
			
		}
		
}

static void locate_BestScore(RESULTS *pResults, int Nrows, int Ncols){

	int row,col;
			
	pResults->BestScore=0;
	
	for(row = 0;  row < Nrows;  row++)
		for( col= 0;  col < Ncols;  col++){
		
			/*
				There is a Higher Score
			*/
			if(Scores(row,col) > pResults->BestScore){
				 pResults->BestScore = Scores(row,col);    /* memorize the best score */
				 pResults->BestScore_row = row;
				 pResults->BestScore_col = col;
			}
			
			
			/*
				The Higher Score is present twice, thus choose as Waterman and Eggert - 1987 - the smallest way
			*/
			if(Scores(row,col) == pResults->BestScore){
				 if(row+col < pResults->BestScore_row+pResults->BestScore_col ||
				      (row+col == pResults->BestScore_row+pResults->BestScore_col &&
						 row<pResults->BestScore_row)  )
				{
				 	pResults->BestScore = Scores(row,col);  
				 	pResults->BestScore_row = row;
				 	pResults->BestScore_col = col;
					
				 }
			}		
		}

}

void waterman_eggert(char *seq1, char *seq2, SCORING *pScoring, RESULTS *pResults, int round,  short verbose){

	int size1= (int) strlen(seq1)+1,
		 size2= (int) strlen(seq2)+1;

	if(round == 0){
		fill_matrices(seq1, seq2, pScoring,pResults);
		traceback_SW( seq1, seq2, pScoring, pResults , 0);
	}
	else{
		refill_matrices(seq1, seq2, pScoring,pResults);
		locate_BestScore(pResults, size1, size2);
		traceback_SW( seq1, seq2, pScoring, pResults , 0);		
	}
	
	if(verbose){
		print_Scoring(*pScoring);
		print_matrices(*pResults, seq1, seq2);
	}
	
	return;
}



